@@ -183,19 +183,7 @@ When live editing the filter, it is bound to :live.")
                       unread-count entry-count
                       (hash-table-count feeds))))))
 
-(defvar elfeed-search--header-cache nil
-  "Cache of the last computed header.")
-
 (defun elfeed-search--header ()
-  "Returns the string to be used as the Elfeed header."
-  (if (eql (car elfeed-search--header-cache) (buffer-modified-tick))
-      (cdr elfeed-search--header-cache)
-    (let* ((header (elfeed-search--header-1))
-           (cache (cons (buffer-modified-tick) header)))
-      (prog1 header
-        (setf elfeed-search--header-cache cache)))))
-
-(defun elfeed-search--header-1 ()
   "Computes the string to be used as the Elfeed header."
   (cond
    ((zerop (elfeed-db-last-update))
@@ -380,7 +368,8 @@ The customization `elfeed-search-date-format' sets the formatting."
         (matches ())
         (not-matches ())
         (limit nil)
-        (feeds ()))
+        (feeds ())
+        (not-feeds ()))
     (cl-loop for element in (split-string filter)
              for type = (aref element 0)
              do (cl-case type
@@ -407,6 +396,9 @@ The customization `elfeed-search-date-format' sets the formatting."
                   (?= (let ((re (substring element 1)))
                         (when (elfeed-valid-regexp-p re)
                           (push re feeds))))
+                  (?~ (let ((re (substring element 1)))
+                        (when (elfeed-valid-regexp-p re)
+                          (push re not-feeds))))
                   (otherwise (when (elfeed-valid-regexp-p element)
                                (push element matches)))))
     `(,@(when before
@@ -424,7 +416,9 @@ The customization `elfeed-search-date-format' sets the formatting."
       ,@(when limit
           (list :limit limit))
       ,@(when feeds
-          (list :feeds feeds)))))
+          (list :feeds feeds))
+      ,@(when not-feeds
+          (list :not-feeds not-feeds)))))
 
 (defun elfeed-search--recover-time (seconds)
   "Pick a reasonable filter representation for SECONDS."
@@ -459,14 +453,12 @@ AFTER-SECONDS and BEFORE-SECONDS."
 The time (@n-units-ago) filter may not exactly match the
 original, but will be equal in its effect."
   (let ((output ()))
-    (let ((after (plist-get filter :after))
-          (before (plist-get filter :before))
-          (must-have (plist-get filter :must-have))
-          (must-not-have (plist-get filter :must-not-have))
-          (matches (plist-get filter :matches))
-          (not-matches (plist-get filter :not-matches))
-          (limit (plist-get filter :limit))
-          (feeds (plist-get filter :feeds)))
+    (cl-destructuring-bind (&key after     before
+                                 must-have must-not-have
+                                 matches   not-matches
+                                 feeds     not-feeds
+                                 limit &allow-other-keys)
+        filter
       (when after
         (push (elfeed-search--recover-units after before) output))
       (dolist (tag must-have)
@@ -481,6 +473,8 @@ original, but will be equal in its effect."
         (push (format "#%d" limit) output))
       (dolist (feed feeds)
         (push (format "=%s" feed) output))
+      (dolist (feed not-feeds)
+        (push (format "~%s" feed) output))
       (mapconcat #'identity (nreverse output) " "))))
 
 (defun elfeed-search-filter (filter entry feed &optional count)
@@ -492,13 +486,11 @@ filtering against a limit filter (ex. #10).
 See `elfeed-search-set-filter' for format/syntax documentation.
 This function must *only* be called within the body of
 `with-elfeed-db-visit' because it may perform a non-local exit."
-  (let ((after (plist-get filter :after))
-        (must-have (plist-get filter :must-have))
-        (must-not-have (plist-get filter :must-not-have))
-        (matches (plist-get filter :matches))
-        (not-matches (plist-get filter :not-matches))
-        (limit (plist-get filter :limit))
-        (feeds (plist-get filter :feeds)))
+  (cl-destructuring-bind (&key must-have must-not-have
+                               matches   not-matches
+                               feeds     not-feeds
+                               after limit &allow-other-keys)
+      filter
     (let* ((tags (elfeed-entry-tags entry))
            (date (elfeed-entry-date entry))
            (age (- (float-time) date))
@@ -527,25 +519,27 @@ This function must *only* be called within the body of
                (cl-some (lambda (f)
                           (or (string-match-p f feed-id)
                               (string-match-p f feed-title)))
-                        feeds))))))
+                        feeds))
+           (cl-notany (lambda (f)
+                        (or (string-match-p f feed-id)
+                            (string-match-p f feed-title)))
+                      not-feeds)))))
 
 (defun elfeed-search-compile-filter (filter)
   "Compile FILTER into a lambda function for `byte-compile'.
 
 Executing a filter in bytecode form is generally faster than
 \"interpreting\" the filter with `elfeed-search-filter'."
-  (let ((after (plist-get filter :after))
-        (before (plist-get filter :before))
-        (must-have (plist-get filter :must-have))
-        (must-not-have (plist-get filter :must-not-have))
-        (matches (plist-get filter :matches))
-        (not-matches (plist-get filter :not-matches))
-        (limit (plist-get filter :limit))
-        (feeds (plist-get filter :feeds)))
+  (cl-destructuring-bind (&key after     before
+                               must-have must-not-have
+                               matches   not-matches
+                               feeds     not-feeds
+                               limit &allow-other-keys)
+      filter
     `(lambda (,(if (or after matches not-matches must-have must-not-have)
                    'entry
                  '_entry)
-              ,(if feeds
+              ,(if (or feeds not-feeds)
                    'feed
                  '_feed)
               ,(if limit
@@ -560,7 +554,7 @@ Executing a filter in bytecode form is generally faster than
                   '((title (or (elfeed-meta entry :title)
                                (elfeed-entry-title entry)))
                     (link (elfeed-entry-link entry))))
-              ,@(when feeds
+              ,@(when (or feeds not-feeds)
                   '((feed-id (elfeed-feed-id feed))
                     (feed-title (or (elfeed-meta feed :title)
                                     (elfeed-feed-title feed) "")))))
@@ -586,6 +580,12 @@ Executing a filter in bytecode form is generally faster than
                            for regex in feeds
                            collect `(string-match-p ,regex feed-id)
                            collect `(string-match-p ,regex feed-title)))))
+              ,@(when not-feeds
+                  `((not
+                     (or ,@(cl-loop
+                            for regex in not-feeds
+                            collect `(string-match-p ,regex feed-id)
+                            collect `(string-match-p ,regex feed-title))))))
               ,@(when before
                   `((> age ,before))))))))
 
@@ -706,8 +706,10 @@ When FORCE is non-nil, redraw even when the database hasn't changed."
           (dolist (entry elfeed-search-entries)
             (funcall elfeed-search-print-entry-function entry)
             (insert "\n"))
-          (insert "End of entries.\n")
           (setf elfeed-search-last-update (float-time))))
+      (when (zerop (buffer-size))
+        ;; If nothing changed, force a header line update
+        (force-mode-line-update))
       (run-hooks 'elfeed-search-update-hook))))
 
 (defun elfeed-search-fetch (prefix)
